@@ -12,9 +12,7 @@
 
 (use-modules (lang c pgen))
 (use-modules (nyacc lalr))
-(use-modules (nyacc lex))
 (use-modules (nyacc export))
-(use-modules (sxml simple))
 (use-modules (ice-9 pretty-print))
 
 (with-output-to-file "lang/c/lang.txt.new"
@@ -24,14 +22,14 @@
 (with-output-to-file "lang/c/gram.y.new"
   (lambda () (lalr->bison clang-spec)))
 (write-lalr-tables clang-mach "lang/c/tables.scm.new")
+(write-lalr-actions clang-mach "lang/c/actions.scm.new")
 
 (define defs '((arch . "x86_64")))
 (define incs '("." "lang/c"))
 
 (let ((sx (with-input-from-file "lang/c/ex1.c"
-	    (lambda () (parse-c #:cpp-defs defs #:inc-dirs incs)))))
+	    (lambda () (dev-parse-c #:cpp-defs defs #:inc-dirs incs)))))
   (pretty-print sx)
-  ;;(sxml->xml sx) (newline)
   #f)
 
 ;; --- last line
