@@ -2,45 +2,33 @@
 ;;;
 ;;; Copyright (C) 2015 Matthew R. Wette
 ;;;
-;;; This library is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU Lesser General Public
-;;; License as published by the Free Software Foundation; either
-;;; version 3 of the License, or (at your option) any later version.
+;;; This program is free software: you can redistribute it and/or modify
+;;; it under the terms of the GNU General Public License as published by 
+;;; the Free Software Foundation, either version 3 of the License, or 
+;;; (at your option) any later version.
 ;;;
-;;; This library is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; Lesser General Public License for more details.
+;;; This program is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;;; GNU General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU Lesser General Public
-;;; License along with this library; if not, write to the Free Software
-;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+;;; You should have received a copy of the GNU General Public License
+;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-;; todo/idea/bugs
+;; C parser generator: based on ISO-C99; with comments and CPP statements
 
 (define-module (lang c pgen)
-  #:export (clang-spec clang-mach clang-lexer clang-parser parse-c parse-cee)
+  #:export (clang-spec clang-mach dev-parse-c)
+  #:use-module (lang c cpp)
   #:use-module (nyacc lalr)
   #:use-module (nyacc lex)
   #:use-module (lang util)
-  #:use-module ((srfi srfi-1) #:select (last remove))
   #:use-module ((srfi srfi-9) #:select (define-record-type))
+  #:use-module ((srfi srfi-43) #:select (vector-map))
   #:use-module ((sxml fold) #:select (foldts*-values foldts))
   #:use-module ((sxml xpath) #:select (sxpath))
-  ;; debug:
-  ;;#:re-export (read-c-ident)
-  #:use-module (ice-9 pretty-print)
   )
 
-(define (fmterr fmt . args)
-  (apply simple-format (current-error-port) fmt args))
-
-(define crn "Copyright (C) 2015 Matthew R. Wette
-
-This software is covered by the GNU GENERAL PUBLIC LICENCE, Version 3,
-or any later version published by the Free Software Foundation.  See the
-file COPYING included with the nyacc distribution.")
-
 ;; Objective is to generate a sxml tree.
 ;; Strategy for building the tree:
 ;; @itemize
@@ -51,8 +39,8 @@ file COPYING included with the nyacc distribution.")
 ;; but modified to handle comments and CPP statements.
 (define clang-spec
   (lalr-spec
-   (notice crn)
-   (expect 1)				; expect 1 shift-reduce conflict
+   (notice lang-crn)
+   (expect 25)				; else plus tons from type-spec
    (start translation-unit-proxy)
    (grammar
 
@@ -62,13 +50,12 @@ file COPYING included with the nyacc distribution.")
     (declaration
      (declaration-specifiers
       initialized-declarator-list
-      ($$ (let ((decl `(decl ,(tl->list $1) ,(tl->list $2))))
-	    (for-each add-typename (find-new-typenames decl))
-	    decl))
+      ($$ (save-typenames `(decl ,(tl->list $1) ,(tl->list $2))))
       ";" opt-code-comment
       ($$ (if (pair? $5) (append $3 (list $5)) $3)))
      )
 
+    ;; At most one storage class specifier and one type specifier may appear.
     (declaration-specifiers
      ;; storage-class-specifier declaration-specifiers_opt
      (storage-class-specifier
@@ -82,9 +69,9 @@ file COPYING included with the nyacc distribution.")
 		     ($$ (tl-insert $2 $1)))
      ;; type-qualifier declaration-specifiers_opt
      (type-qualifier
-      ($$ (make-tl 'decl-spec-list `(type-qual ,$1))))
+      ($$ (make-tl 'decl-spec-list $1)))
      (type-qualifier declaration-specifiers
-		     ($$ (tl-insert $2 `(type-qual ,$1))))
+		     ($$ (tl-insert $2 $1)))
      ;; function-specifier declaration-specifiers_opt
      (function-specifier
       ($$ (make-tl 'decl-spec-list `(fctn-spec ,$1))))
@@ -117,13 +104,13 @@ file COPYING included with the nyacc distribution.")
 
     ;; 4.4, p 86
     (type-specifier
-     (enumeration-type-specifier)
-     (floating-point-type-specifier)
-     (integer-type-specifier)
-     (structure-type-specifier)
-     (typedef-name)
-     (union-type-specifier)
-     (void-type-specifier)
+     (enumeration-type-specifier ($$ `(type-spec ,$1)))
+     (floating-point-type-specifier ($$ `(type-spec ,$1)))
+     (integer-type-specifier ($$ `(type-spec ,$1)))
+     (structure-type-specifier ($$ `(type-spec ,$1)))
+     (typedef-name ($$ `(type-spec ,$1)))
+     (union-type-specifier ($$ `(type-spec ,$1)))
+     (void-type-specifier ($$ `(type-spec ,$1)))
      )
 
     (type-qualifier
@@ -152,14 +139,14 @@ file COPYING included with the nyacc distribution.")
 
     ;; 4.5.2, p 96
     (pointer-declarator
-     (pointer direct-declarator ($$ `(pointer-declr ,$1 ,$2)))
+     (pointer direct-declarator ($$ `(ptr-declr ,$1 ,$2)))
      )
 
     (pointer
      ("*" type-qualifier-list ($$ `(pointer ,$2)))
      ("*" ($$ '(pointer)))
      ("*" type-qualifier-list pointer ($$ `(pointer ,(tl->list $2) ,$3)))
-     ("*" pointer ($$ `(pointer ,$1)))
+     ("*" pointer ($$ `(pointer ,$2)))
      )
 
     (type-qualifier-list ; (C89)
@@ -180,16 +167,16 @@ file COPYING included with the nyacc distribution.")
      (direct-declarator
       "[" array-size-expression "]"
       ($$ `(array-of ,$1 ,$3)))
-      (direct-declarator
-       "[" "]"
-       ($$ `(array-of ,$1)))
-      (direct-declarator
-       "[" array-qualifier-list "*" "]"
-       ($$ `(array-of ,$1 ,$3 "*???")))
-      (direct-declarator
-       "[" "*" "]"
-       ($$ `(array-of ,$1 "*???")))
-      )
+     (direct-declarator
+      "[" "]"
+      ($$ `(array-of ,$1)))
+     (direct-declarator
+      "[" array-qualifier-list "*" "]"
+      ($$ `(array-of ,$1 ,$3 "*???")))
+     (direct-declarator
+      "[" "*" "]"
+      ($$ `(array-of ,$1 "*???")))
+     )
 
     (array-qualifier-list
      (array-qualifier ($$ (make-tl 'array-qual-list (list $1))))
@@ -200,12 +187,12 @@ file COPYING included with the nyacc distribution.")
      ("static") ("restrict") ("const") ("volatile"))
 
     (array-size-expression
-     ;; I think the following may be an error:
-     ;; It conflicts with xxx which uses constant-expression
+     ;; I don't know how to handle 'constant-expression here.  It generates
+     ;; a conflict with (to be documented).
+     ;;(constant-expression) =>
      (assignment-expression)
-     ;;(constant-expression)
-     ;; I think the following may be an error:
-     ;; it is handled by array-declarator.
+     ;; I don't know how to deal with "*" here.  It seems to be handled
+     ;; by 'array-declarator.
      ;;("*")
      )
 
@@ -213,13 +200,13 @@ file COPYING included with the nyacc distribution.")
     (function-declarator
      (direct-declarator
       "(" parameter-type-list ")"
-      ($$ `(fctn-declr ,$1 ,(tl->list $3))))
+      ($$ `(ftn-declr ,$1 ,(tl->list $3))))
      (direct-declarator
       "(" identifier-list ")"
-      ($$ `(fctn-declr ,$1 ,(tl->list $3))))
+      ($$ `(ftn-declr ,$1 ,(tl->list $3))))
      (direct-declarator
       "(" ")"
-      ($$ `(fctn-declr ,$1)))
+      ($$ `(ftn-declr ,$1)))
      )
 
     (parameter-type-list
@@ -269,7 +256,7 @@ file COPYING included with the nyacc distribution.")
 
     (designator-list
      (designator ($$ (make-tl 'desgr-list $1)))
-     (designator-list designator ($$ (tl->append $1 $2)))
+     (designator-list designator ($$ (tl-append $1 $2)))
      )
 
     (designator
@@ -288,93 +275,63 @@ file COPYING included with the nyacc distribution.")
     ;; 5.1.1, p 125
     ;; The productions shown in the book are not LALR1 so we will need to use
     ;; static semantics.
-    (signed-type-specifier
-     ("short" ($$ '(short)))
-     ("int" ($$ '(int)))
-     ("signed" ($$ '(int)))
-     ("long" ($$ '(long)))
-     )
-    #| 
     (signed-type-specifier ;; Enforce with static semantics.
-     ("short" ($$ '(signed (short-int))))
-     ("short" "int" ($$ '(signed (short-int))))
-     ("signed" "short" ($$ '(signed (short-int))))
-     ("signed" "short" "int" ($$ '(signed (short-int))))
-     ("int" ($$ '(signed (int))))
-     ("signed" ($$ '(signed (int))))
-     ("signed" "int" ($$ '(signed (int))))
-     ("long" ($$ '(signed (long-int))))
-     ("long" "int" ($$ '(signed (long-int))))
-     ("signed" "long" ($$ '(signed (long-int))))
-     ("signed" "long" "int" ($$ '(signed (long-int))))
-     ("long" "long" ($$ '(signed (long-long-int))))
-     ("long" "long" "int" ($$ '(signed (long-long-int))))
-     ("signed" "long" "long" ($$ '(signed (long-long-int))))
-     ("signed" "long" "long" "int" ($$ '(signed (long-long-int))))
+     ("short" ($$ '(fixed "short")))
+     ("short" "int" ($$ '(fixed "short int")))
+     ("signed" "short" ($$ '(fixed "signed short")))
+     ("signed" "short" "int" ($$ '(fixed "signed short int")))
+     ("int" ($$ '(fixed "int")))
+     ("signed" ($$ '(fixed "signed")))
+     ("signed" "int" ($$ '(fixed "signed int")))
+     ("long" ($$ '(fixed "long")))
+     ("long" "int" ($$ '(fixed "long int")))
+     ("signed" "long" ($$ '(fixed "signed long")))
+     ("signed" "long" "int" ($$ '(fixed "signed long int")))
+     ("long" "long" ($$ '(fixed "long long")))
+     ("long" "long" "int" ($$ '(fixed "long long int")))
+     ("signed" "long" "long" ($$ '(fixed "singed long long")))
+     ("signed" "long" "long" "int" ($$ '(fixed "signed long long int")))
      )
-    |#
 
     ;; 5.1.2, p 128
     (unsigned-type-specifier
-     ("unsigned" ($$ '(unsigned)))
+     ("unsigned" "short" "int" ($$ '(fixed "unsigned short int")))
+     ("unsigned" "short" ($$ '(fixed "unsinged short")))
+     ("unsigned" "int" ($$ '(fixed "unsigned int")))
+     ("unsigned" ($$ '(fixed "unsigned")))
+     ("unsigned" "long" "int" ($$ '(fixed "unsigned long")))
+     ("unsigned" "long" ($$ '(fixed "unsigned long")))
+     ("unsigned" "long" "long" "int" ($$ '(fixed "unsigned long long int")))
+     ("unsigned" "long" "long" ($$ '(fixed "unsigned long long")))
      )
-    #|
-    (unsigned-type-specifier
-     ("unsigned" "short" "int" ($$ '(unsigned (short-int))))
-     ("unsigned" "short" ($$ '(unsigned (short-int))))
-     ("unsigned" "int" ($$ '(unsigned (int))))
-     ("unsigned" ($$ '(unsigned (int))))
-     ("unsigned" "long" "int" ($$ '(unsigned (long-int))))
-     ("unsigned" "long" ($$ '(unsigned (long-int))))
-     ("unsigned" "long" "long" "int" ($$ '(unsigned (long-long-int))))
-     ("unsigned" "long" "long" ($$ '(unsigned (long-long-int))))
-     )
-    |#
 
     ;; 5.1.3, p 129
     (character-type-specifier ;; Enforce with static semantics.
-     ("char" ($$ '(char)))
+     ("char" ($$ '(fixed "char")))
+     ("signed" "char" ($$ '(fixed "signed char")))
+     ("unsigned" "char" ($$ '(fixed "unsigned char")))
      )
-    #|
-    (character-type-specifier ;; Enforce with static semantics.
-     ("char" ($$ '(char)))
-     ("signed" "char" ($$ '(signed (char))))
-     ("unsigned" "char" ($$ '(unsigned (char))))
-     )
-    |#
 
     ;; 5.1.5, p 132, discussed but not defined
     (bool-type-specifier
-     ("_Bool" ($$ '(bool)))
+     ("_Bool" ($$ '(fixed "_Bool")))
      )
 
     ;; 5.2, p 132
     (floating-point-type-specifier
-     ("float" ($$ '(float)))
-     ("double" ($$ '(double)))
+     ("float" ($$ '(float "float")))
+     ("double" ($$ '(float "double")))
+     ("long" "double" ($$ '(float "long double")))
      (complex-type-specifier)
      )
-    #|
-    (floating-point-type-specifier
-     ("float" ($$ '(float)))
-     ("double" ($$ '(double)))
-     ("long" "double" ($$ '(long-double)))
-     (complex-type-specifier)
-     )
-    |#
 
     ;; 5.2.1, p 136
     (complex-type-specifier
-     ("_Complex" ($$ '(complex)))
+     ("_Complex" ($$ '(complex "_Complex")))
+     ("float" "_Complex" ($$ '(complex "float _Complex")))
+     ("double" "_Complex" ($$ '(complex "double _Complex")))
+     ("long" "double" "_Complex" ($$ '(complex "long double _Complex")))
      )
-    #|
-    (complex-type-specifier
-     ("_Complex" ($$ '(complex)))
-     ("float" "_Complex" ($$ '(complex (float))))
-     ("double" "_Complex" ($$ '(complex (double))))
-     ("long" "double" "_Complex" ($$ '(complex (long-double))))
-     )
-    |#
 
     ;; 5.5, p 145
     (enumeration-type-specifier
@@ -386,11 +343,11 @@ file COPYING included with the nyacc distribution.")
      ("enum" enumeration-tag "{" enumeration-definition-list "}"
       ($$ `(enum-def ,$1 ,(tl->list $4))))
      ("enum" "{" enumeration-definition-list "}"
-      ($$ `(enum-def ,(tl->list $4))))
+      ($$ `(enum-def ,(tl->list $3))))
      ("enum" enumeration-tag "{" enumeration-definition-list "," "}"
       ($$ `(enum-def ,$1 ,(tl->list $4))))
      ("enum" "{" enumeration-definition-list "," "}"
-      ($$ `(enum-def ,(tl->list $4))))
+      ($$ `(enum-def ,(tl->list $3))))
      )
     
     (enumeration-type-reference
@@ -407,10 +364,10 @@ file COPYING included with the nyacc distribution.")
 
     (enumeration-constant-definition
      (enumeration-constant ($$ `(enum-defn ,$1)))
-     ;; I think this may be an error.  It's replacement follows.
-     ;; If expression, then comma-expressions are allowed but enum uses
-     ;; commas in list of enumerations.
-     ;;(enumeration-constant "=" expression)  ;;  --[change to]--> 
+     ;; I'm not not sure how to get this working. With 'expression, then
+     ;; comma-expressions are allowed but enum uses commas in list of
+     ;; enumerations.  So I'm using 'constant-expression instead for now.
+     ;; (enumeration-constant "=" expression)  ;;  --[change to]--> 
      (enumeration-constant "=" constant-expression
 			   ($$ `(enum-defn ,$1 ,$3)))
      )
@@ -486,11 +443,11 @@ file COPYING included with the nyacc distribution.")
     (union-tag (identifier))
 
     ;; 5.9, p 168
-    (void-type-specifier ("void"))
+    (void-type-specifier ("void" ($$ '(void))))
 
     ;; 5.10, p 168
     ;;(typedef-name (identifier)) must be hacked w/ the lexical analyzer
-    (typedef-name ('typename ($$ `(ident ,$1))))
+    (typedef-name ('typename ($$ `(typename ,$1))))
 
     ;; 5.12, p 176
     (type-name
@@ -502,20 +459,20 @@ file COPYING included with the nyacc distribution.")
      )
 
     (abstract-declarator
-     (pointer)
+     (pointer ($$ `(abs-declr ,$1)))
      (pointer direct-abstract-declarator ($$ `(abs-declr ,$1 ,$2)))
-     (direct-abstract-declarator)
+     (direct-abstract-declarator ($$ `(abs-declr ,$1)))
      )
 
     ;; The following in 4.5.2
     ;; pointer =>
     ;; type-qualifier-list => 
 
-    ;; I have removed the constant-expression productions below because
-    ;; these will conflict with expression which expands to constant-expr.
+    ;; I have removed 'constant-expression productions below because
+    ;; these conflict with expression which expands to constant-expression.
     (direct-abstract-declarator
      ("(" abstract-declarator ")" ($$ `(declr-scope ,$2)))
-     ;;(direct-abstract-declarator "[" constant-expression "]")
+     ;;(direct-abstract-declarator "[" constant-expression "]") 
      (direct-abstract-declarator "[" "]" ($$ `(declr-array ,$1)))
      ;;("[" constant-expression "]")
      ("[" "]")
@@ -855,9 +812,9 @@ file COPYING included with the nyacc distribution.")
 
     (for-expressions
      ("(" initial-clause expression ";" expression ")" ($$ (list $2 $3 $5)))
-     ("(" initial-clause expression ";" ")" ($$ (list $2 $3 (expr))))
-     ("(" initial-clause ";" expression ")" ($$ (list $2 (expr) $4)))
-     ("(" initial-clause ";" ")" ($$ (list $2 (expr) (expr))))
+     ("(" initial-clause expression ";" ")" ($$ (list $2 $3 '(expr))))
+     ("(" initial-clause ";" expression ")" ($$ (list $2 '(expr) $4)))
+     ("(" initial-clause ";" ")" ($$ (list $2 '(expr) '(expr))))
      )
     
     (initial-clause
@@ -898,7 +855,7 @@ file COPYING included with the nyacc distribution.")
 
     ;; 8.11, p 281
     (null-statement
-     (";" ($$ (null-stmt))))
+     (";" ($$ '(null-stmt))))
 
     ;; 9.1, p 286
     (translation-unit
@@ -960,364 +917,6 @@ file COPYING included with the nyacc distribution.")
     (cpp-statement ('cpp-stmt ($$ `(cpp-stmt ,$1))))
     )))
 
-
-;; ------------------------------------------------------------------------
-;; cpp devel
-
-#|
-  #define  #undef  #include  #if  #ifdef  #ifndef  #else  #endif  #elif
-  #line  defined  # <operator>  ## <operator>  #pragma  #error
-
-  need mini-cpp and full-cpp
-  mini-cpp:
-  don't expand macro calls -- treat like function calls, but provide dict
-  if ifdef args not defined hmmm -- for now make sure they are
-     ... need script to get #defines from cc
-  provide dict of #defines
-|#
-
-(define (skip-ws ch)
-  (if (eof-object? ch) ch
-      (if (char-set-contains? c:ws ch)
-	  (skip-ws (read-char))
-	  ch)))
-
-;; grammar:
-;; expr   => expr == equal | equal
-;; equal  => equal || term | term
-;; term   => term && factor | factor
-;; factor => defined '(' ident ')' | const
-;; TO
-;; expr => equal expr'
-;; expr' => == equal | $eps
-;; equal => term equal'
-;; equal' => || term | $eps
-;; term => factor term'
-;; term' => && factor | $eps
-;; factor => defined | const
-(define (parse-cpp-expr)
-  ;; defined(X) && define(Y) ...
-  (letrec
-      ((p-expr
-	(lambda (la)
-	  (let* ((equal (p-equal la)) (la1 (read-char))
-		 (expr1 (if (eof-object? la1) #f (p-expr1 la1))))
-	    ;;(simple-format #t "p-expr la=~S\n" la)
-	    (if expr1 (cons equal expr1) equal))))
-       (p-expr1
-	(lambda (la)
-	  #f))
-       (p-equal
-	(lambda (la)
-	  (let* ((term (p-term la)) (la1 (if term (read-char) la))
-		 (equal1 (if (eof-object? la1) #f (p-equal1 la1))))
-	    ;;(simple-format #t "p-equal la=~S\n" la)
-	    (if equal1 (cons term equal1) term))))
-       (p-equal1
-	(lambda (la)
-	  #f))
-       (p-term
-	(lambda (la)
-	  (let* ((factor (p-factor la)) (la1 (if factor (read-char) la))
-		 (term1 (if (eof-object? la1) #f (p-term1 la1))))
-	    ;;(simple-format #t "p-term la=~S factor=~S\n" la factor)
-	    (if term1 (cons factor term1) factor))))
-       (p-term1
-	(lambda (la)
-	  #f))
-       (p-factor
-	(lambda (la)
-	  (let ((la1 (skip-ws la)))
-	    (cond
-	     ((p-cnst la1))
-	     ((p-defd la1))
-	     ((read-c-ident la1) => (lambda (id) `(ident ,id)))
-	     (else #f)))))
-       (p-defd		    ; "defined(IDENT)" => '(defined_p "IDENT")
- 	(let ((rd-defd (make-chseq-reader '(("defined" . defined_p)))))
-	  (lambda (la)
-	    (and
-	     (rd-defd la)
-	     (eq? (read-char) #\()
-	     (let ((ident (read-c-ident (read-char))))
-	       (and (eq? (read-char) #\))
-		    `(defined_p ,ident)))))))
-       (p-cnst
-	(lambda (la)
-	  (let ((num (read-c-num la)))
-	    (if num `(num ,(cdr (read-c-num la))) #f))))
-       )
-    (p-expr (read-char))))
-
-;; @item eval-cpp-expr tree dict => value
-;; evaluate a CPP expression
-(define (eval-cpp-expr tree dict)
-  (let* ()
-    (case (car tree)
-      ((num)
-       (string->number (cadr tree)))
-      ((ident) ;; ref
-       (let* ((repl (assoc-ref dict (cadr tree))) ; replacement
-	      (tree (and repl (with-input-from-string repl parse-cpp-expr)))
-	      (value (and tree (eval-cpp-expr tree dict))))
-	 ;; returns value of #f if not ident not defined
-	 value))
-      ((not)
-       (let ((arg (eval-cpp-expr (cadr tree) dict)))
-	 (if (eq? arg #f) #f
-	     (if (zero? arg) 1 0))))
-      ((defined_p)
-       (let ((ident (string->symbol (cadr tree))))
-	 (if (assq-ref dict ident) 1 0)))
-      (else #f))))
-
-;; @item cpp-define => #f|???
-(define (cpp-define)
-  ;; The (broken) parse architecture is "unread la argument if no match"
-  (letrec
-      ((p-cppd ;; parse all
-	(lambda ()
-	  (let* ((iden (read-c-ident (skip-ws (read-char))))
-		 (args (or (p-args (skip-ws (read-char))) '()))
-		 (amap (lambda (as) (map (lambda (a) (list 'arg a)) as)))
-		 (rest (or (p-rest (skip-ws (read-char))) " ")))
-	    (if (pair? args)
-		`(define (name ,iden) ,(cons 'args (amap args)) (repl ,rest))
-		`(define (name ,iden) (repl ,rest))))))
-       (p-args ;; parse args
-	(lambda (la) ;; unread la if no match :(
-	  (if (eq? la #\()
-	      (let iter ((args '()) (la (skip-ws (read-char))))
-		(cond
-		 ((eq? la #\)) (reverse args))
-		 ((read-c-ident la) =>
-		  (lambda (arg) (iter (cons arg args) (skip-ws (read-char)))))
-		 ((eq? la #\,)
-		  (iter args (skip-ws (read-char))))))
-	      (begin (if (char? la) (unread-char la)) #f)))) ;; CLEANUP
-       (p-rest ;; parse rest
-	(lambda (la)
-	  (cond ((char? la) (unread-char la) (drain-input (current-input-port)))
-		(else #f)))))
-    (p-cppd)))
-
-(define (cpp-include)
-  (let* ((beg-ch (skip-ws (read-char)))
-	 (end-ch (if (eq? beg-ch #\<) #\> #\"))
-	 (path (let iter ((cl (list beg-ch)) (ch (read-char)))
-		 (if (eq? ch end-ch) (list->string (reverse (cons ch cl)))
-		     (iter (cons ch cl) (read-char))))))
-    `(include ,path)))
-
-(define (parse-cpp-line line)
-  (with-input-from-string line
-    (lambda ()
-      (let ((cmd (string->symbol (read-c-ident (skip-ws (read-char)))))
-	    (rd-ident (lambda () (read-c-ident (skip-ws (read-char))))))
-	(cons
-	 'cpp-stmt
-	 (case cmd
-	   ((include) (cpp-include))
-	   ((ifdef) `(if (defined_p ,(rd-ident))))
-	   ((ifndef) `(if (not (defined_p ,(rd-ident)))))
-	   ((define) (cpp-define))
-	   ((if elseif) (list cmd (parse-cpp-expr)))
-	   ((else endif) (list cmd))
-	   (else '())))))))
-    
-;; @item read-cpp-line ch => #f | (cpp-xxxx)??
-;; Given if ch is #\# read a cpp-statement
-(define (read-cpp-line ch)
-  (if (not (eq? ch #\#)) #f
-      (let iter ((cl '()) (ch (read-char)))
-	(cond
-	 ((eq? ch #\newline) (list->string (reverse cl)))
-	 ((eq? ch #\\)
-	  (let ((c2 (read-char)))
-	    (if (eq? c2 #\newline)
-		(iter cl (read-char))
-		(iter (cons* c2 ch cl) (read-char)))))
-	 (else (iter (cons ch cl) (read-char)))))))
-
-
-;; ------------------------------------------------------------------------
-
-(define-record-type cpi
-  (make-cpi-1)
-  cpi?
-  (defines cpi-defs set-cpi-defs!)
-  (incdirs cpi-incs set-cpi-incs!)
-  (typnams cpi-tyns set-cpi-tyns!)
-  )
-
-(define (make-cpi defines incdirs)
-  (let* ((cpi (make-cpi-1)))
-    (set-cpi-defs! cpi (if defines defines '()))
-    (set-cpi-incs! cpi (if incdirs incdirs '()))
-    (set-cpi-tyns! cpi '())
-    cpi))
-
-(define *info* (make-fluid #f))
-
-;; @item typename? name
-;; Called by lexer to determine if symbol is a typename.
-(define (typename? name)
-  (let ((info (fluid-ref *info*)))
-    (memq name (cpi-tyns info))))
-
-;; @item add-typename name
-;; Called by parser to tell lexer this is a new typename.
-(define (add-typename name)
-  ;;(simple-format #t "add-typename: ~S\n" name)
-  (let ((info (fluid-ref *info*)))
-    (set-cpi-tyns! info (cons (string->symbol name) (cpi-tyns info)))))
-
-;; @item add-typenames-from-decl
-;; This finds typenames using @code{find-new-typenames} and adds via
-;; @code{add-typename}.
-(define (add-typenames-from-decl decl)
-  (for-each add-typename (find-new-typenames decl)))
-
-(use-modules (ice-9 pretty-print))
-
-;; @item find-new-typenames decl
-;; Given declaration return a list of new typenames (via @code{typedef}).
-(define find-new-typenames
-  (let ((sxtd (sxpath '(stor-spec typedef)))
-	(sxid (sxpath '(init-declr ident *text*))))
-    (lambda (decl)
-      (cond
-       ((not (eq? 'decl (car decl))) '())
-       ((< (length decl) 3) '())
-       (else (let* ((spec-list (list-ref decl 1))
-		    (init-list (list-ref decl 2)))
-	       ;;(simple-format #t "\nelse:\n")
-	       ;;(pretty-print spec-list)
-	       ;;(pretty-print init-list)
-	       ;;(simple-format #t "\nsxtd=>~S\n" (sxtd spec-list))
-	       ;;(simple-format #t "\nsxid=>~S\n" (sxid init-list))
-	       (if (pair? (sxtd spec-list)) (sxid init-list) '())))))))
-
-
-;; ------------------------------------------------------------------------
-
-;; @item find-file-in-dirl file dirl => path
-(define (find-file-in-dirl file dirl)
-  (let iter ((dirl dirl))
-    (if (null? dirl) #f
-	(let ((p (string-append (car dirl) "/" file)))
-	  (if (access? p R_OK) p (iter (cdr dirl)))))))
-
-;; @item make-c-lexer match-table => proc
-;; Context-sensitive lexer for the C language.
-;; This gets ugly in order to handle cpp.
-;;.need to add support for num's w/ letters like @code{14L} and @code{1.3f}.
-;; todo: I think there is a bug wrt the comment reader because // ... \n will
-;; end up in same mode...  so after
-;; int x; // comment
-;; the lexer will think we are not at BOL.
-(define (make-c-lexer-generator match-table)
-  (let* ((read-ident read-c-ident)
-	 (read-comm read-c-comm)
-	 ;;
-	 (ident-like? (make-ident-like-p read-ident))
-	 ;;
-	 (strtab (filter-mt string? match-table)) ; strings in grammar
-	 (kwstab (filter-mt ident-like? strtab))  ; keyword strings =>
-	 (keytab (map-mt string->symbol kwstab))  ; keywords in grammar
-	 (chrseq (remove-mt ident-like? strtab))  ; character sequences
-	 (symtab (filter-mt symbol? match-table)) ; symbols in grammar
-	 (chrtab (filter-mt char? match-table))	  ; characters in grammar
-	 ;;
-	 (read-chseq (make-chseq-reader chrseq))
-	 (assc-$ (lambda (pair) (cons (assq-ref symtab (car pair)) (cdr pair))))
-	 ;;
-	 (t-ident (assq-ref symtab '$ident))
-	 (t-typename (assq-ref symtab 'typename))
-	 (xp1 (sxpath '(cpp-stmt define)))
-	 (xp2 (sxpath '(decl))))
-    (lambda ()
-      (let ((bol #t)		       ; begin-of-line condition
-	    (skip (list #f))	       ; CPP skip-input stack
-	    (info (fluid-ref *info*))) ; assume make and run in same thread
-	;; Return the first (tval lval) pair not excluded by the CPP.
-	(lambda ()
-      
-	  (define (add-define tree)
-	    (let* ((tail (cdr tree))
-		   (name (string->symbol (car (assq-ref tail 'name))))
-		   (args (assq-ref tail 'args))
-		   (repl (car (assq-ref tail 'repl)))
-		   (cell (cons name (if args (cons args repl) repl))))
-	      (set-cpi-defs! info (cons cell (cpi-defs info)))))
-	  
-	  (define (exec-cpp line)
-	    ;; Parse the line into a CPP stmt, execute it, and return it.
-	    (let* ((stmt (parse-cpp-line line)) (tree (cdr stmt)))
-	      (case (car tree)
-		((include)
-		 (let* ((parg (cadr tree)) (leng (string-length parg))
-			(file (substring parg 1 (1- leng)))
-			(path (find-file-in-dirl file (cpi-incs info)))
-			(tree (with-input-from-file path run-parse)))
-		   ;; (for-each add-typenames-from-decl (xp2 tree))
-		   (for-each add-define (xp1 tree))))
-		((define)
-		 (add-define tree))
-		((if)
-		 (let ((val (eval-cpp-expr (cadr tree) (cpi-defs info))))
-		   (cond ((not val)
-			  (fmterr "*** unresolved: ~S" (cadr tree)))
-			 ((zero? val) (set! skip (cons #t skip)))
-			 (else (set! skip (cons #f skip))))))
-		((endif)
-		 (set! skip (cons 'd skip))))
-	      stmt))
-	  
-	  (define (read-token)
-	    ;;(define (echo lp) (simple-format #t "tok=~S\n" lp) lp)
-	    (define (echo lp) lp)
-	    (echo
-	     (let iter ((ch (read-char)))
-	       (cond
-		((eof-object? ch) (assc-$ '($end . "")))
-		((eq? ch #\newline) (set! bol #t) (iter (read-char)))
-		((char-set-contains? c:ws ch) (iter (read-char)))
-		(bol
-		 (cond
-		  ((read-comm ch) =>
-		   (lambda (c) (assc-$ (cons '$lone-comm (cdr c)))))
-		  ((read-cpp-line ch) => (lambda (s) (assc-$ (exec-cpp s))))
-		  (else (set! bol #f) (iter ch))))
-		((read-ident ch) =>
-		 (lambda (str)
-		   (let ((sym (string->symbol str)))
-		     (cond ((assq-ref keytab sym) => (lambda (t) (cons t str)))
-			   ((typename? sym)
-			    (cons (assq-ref symtab 'typename) str))
-			   (else (cons (assq-ref symtab '$ident) str))))))
-		((read-c-num ch) => assc-$)
-		((read-c-string ch) => assc-$)
-		((read-comm ch) =>
-		 (lambda (c) (assc-$ (cons '$code-comm (cdr c)))))
-		;;((and (simple-format #t "chs=>~S\n" (read-chseq ch)) #f))
-		((read-chseq ch) => identity)
-		((assq-ref chrtab ch) => (lambda (t) (cons t (string ch))))
-		(else (cons ch (string ch))))))
-	    )
-
-	  ;; Loop between reading tokens and skipping tokens.
-	  ;; The use of "delayed pop" is not clean IMO.  Cleaner way?
-	  (let loop ((pair (read-token)))
-	    (case (car skip)
-	      ((#f) pair)
-	      ((#t) (loop (read-token)))
-	      ((d) (let ((fl (cadr skip))) ;; d for endif: delayed pop
-		     (set! skip (cddr skip))
-		     (if fl (loop (read-token)) pair))))))))))
-
-;; ------------------------------------------------------------------------
-
 (define clang-mach
   (compact-machine
    ;;(identity
@@ -1325,23 +924,25 @@ file COPYING included with the nyacc distribution.")
     ;;(identity
     (make-lalr-machine clang-spec))))
 
-(define clang-match-table (assq-ref clang-mach 'mtab))
+(define len-v (assq-ref clang-mach 'len-v))
+(define pat-v (assq-ref clang-mach 'pat-v))
+(define rto-v (assq-ref clang-mach 'rto-v))
+(define mtab (assq-ref clang-mach 'mtab))
+(define sya-v (vector-map
+               (lambda (ix nrg guts) (wrap-action nrg guts))
+               (assq-ref clang-mach 'nrg-v) (assq-ref clang-mach 'act-v)))
+(define act-v (vector-map (lambda (ix f) (eval f (current-module))) sya-v))
 
-;; Parse given a token generator.  Uses fluid @code{*info*}.
-(define parse/tokgen (make-lalr-parser clang-mach))
+(include "pbody.scm")
 
+(define raw-parser (make-lalr-parser clang-mach))
 
-;; Generate a function to provide instances of a token generator.
-(define gen-c-lexer (make-c-lexer-generator (assq-ref clang-mach 'mtab)))
+(define (run-parse) (raw-parser (gen-c-lexer)))
 
-;; Thunk to parser the current input with a new token generator.
-(define (run-parse) (parse/tokgen (gen-c-lexer)))
-
-(define* (parse-c #:key (cpp-defs '()) (inc-dirs '()))
+(define* (dev-parse-c #:key (cpp-defs '()) (inc-dirs '()))
   (let ((info (make-cpi cpp-defs inc-dirs)))
-    (with-fluid* *info* info run-parse)
-    ))
+    (with-fluid* *info* info (lambda () (raw-parser (gen-c-lexer))))))
 
+;(define dev-parse-c parse-c)
 
-;; mode: scheme ***
 ;; --- last line
