@@ -25,29 +25,21 @@
 	    make-lalr-parser
 	    pp-lalr-grammar pp-lalr-machine
 	    write-lalr-tables
+	    write-lalr-actions
+	    ;;write-lalr-parser
 	    wrap-action
+	    pp-rule find-terminal	; used by (nyacc export)
+	    with-gram new-non-kernels
 	    )
-  ;; Clean this up: (e.g., (srfi srfi-1) preferred over (rnrs lists))
-  #:use-module ((rnrs lists) #:select (remq remp remove fold-left fold-right))
   #:use-module ((srfi srfi-1)
-		#:select (lset-intersection lset-union lset-difference fold))
+		#:select (fold fold-right remove
+			       lset-intersection lset-union lset-difference))
   #:use-module ((srfi srfi-9) #:select (define-record-type))
   #:use-module ((srfi srfi-43) #:select (vector-map vector-for-each))
   #:use-module (nyacc util)
   )
 
-(define *nyacc-version* "0.61.0")
-
-#|
-(define-syntax $?
-  (syntax-rules ()
-    ((_ <symb>)
-     (let* ((s (symbol->string (quote <symb>)))
-	    (os (string-append "opt-" s))
-	    (sy (string->symbol os)))
-       `(rule ,sy (,sy () (<symb>)))))))
-|#
-;; maybe?: (rule-macro $* ((_ ssss ($$ ...)
+(define *nyacc-version* "0.62.0")
 
 ;; @item proxy-? sym rhs
 ;; @example
@@ -113,15 +105,7 @@
 (define-syntax lalr-spec
   (syntax-rules +++ () 
     ((_ <expr> +++)
-     (let* (#;(xtra-rules '())		; symbols for optional's
-	    #;(nxsymb 0)			; number of extra symbols
-	    #;(make-symb
-	     (lambda ()
-	       (let ((n nxsymb))
-		 (set! nxsymb (1+ n))
-		 (string->symbol
-		  (string-append "$opt-" (number->string n))))))
-	    )
+     (let* ()
        (letrec-syntax
 	   ((with-attr-list
 	     (syntax-rules ($prune)
@@ -131,11 +115,7 @@
 	    (parse-rhs
 	     (lambda (x)
 	       ;; The following is syntax-case because we use a fender.
-	       (syntax-case x (quote
-			       $$ $$/ref $$-ref
-			       $action $action/ref $action-ref
-			       $with
-			       $? $* $+)
+	       (syntax-case x (quote $$ $$/ref $$-ref $with $? $* $+)
 		 ;; action specifications
 		 ((_ ($$ <guts> ...) <e2> ...)
 		  #'(cons '(action #f #f <guts> ...) (parse-rhs <e2> ...)))
@@ -143,12 +123,6 @@
 		  #'(cons '(action #f '<ref> #f) (parse-rhs <e2> ...)))
 		 ((_ ($$/ref <ref> <guts> ...) <e2> ...)
 		  #'(cons '(action #f <ref> <guts> ...) (parse-rhs <e2> ...)))
-		 ((_ ($action <guts> ...) <e2> ...)
-		  #'(cons '(action #f #f <guts> ...) (parse-rhs <e2> ...)))
-		 ((_ ($action-ref <ref>) <e2> ...)
-		  #'(cons '(action #f '<ref> #f) (parse-rhs <e2> ...)))
-		 ((_ ($action/ref <ref> <guts> ...) <e2> ...)
-		  #'(cons '(action #f <ref> <guts> ...) (parse-rhs <e2> ...)))
 
 		 ;; other internal $-syntax
 		 ((_ ($with <lhs-ref> <ex> ...) <e2> ...)
@@ -166,6 +140,7 @@
 		  #'(cons (cons* 'proxy proxy-* (parse-rhs <s1> <s2> ...))
 			  (parse-rhs <e2> ...)))
 		 
+		 ;; terminals and non-terminals
 		 ((_ (quote <e1>) <e2> ...)
 		  #'(cons '(terminal . <e1>) (parse-rhs <e2> ...)))
 		 ((_ (<f> ...) <e2> ...)
@@ -178,18 +153,16 @@
 		 ((_) #'(list)))))
 	    (parse-rhs-list
 	     (syntax-rules ()
-	       ((_ (<e1> ...) <rhs2> <rhs3> ...)
-		(cons (parse-rhs <e1> ...)
-		      (parse-rhs-list <rhs2> <rhs3> ...)))
-	       ((_ (<e1> ...))
-		(list (parse-rhs <e1> ...)))))
+	       ((_ (<ex> ...) <rhs> ...)
+		(cons (parse-rhs <ex> ...)
+		      (parse-rhs-list <rhs> ...)))
+	       ((_) '())))
 	    (parse-grammar
 	     (syntax-rules ()
-	       ((_ (<lhs> <rhs1> ...) <prod2> <prod3> ...)
-		(cons (cons '<lhs> (parse-rhs-list <rhs1> ...))
-		      (parse-grammar <prod2> <prod3> ...)))
-	       ((_ (<lhs> <rhs1> ...))
-		(list (cons '<lhs> (parse-rhs-list <rhs1> ...))))))
+	       ((_ (<lhs> <rhs> ...) <prod> ...)
+		(cons (cons '<lhs> (parse-rhs-list <rhs> ...))
+		      (parse-grammar <prod> ...)))
+	       ((_) '())))
 	    (parse-precedence
 	     (syntax-rules (left right nonassoc)
 	       ((_ (left <tk> ...) <ex> ...)
@@ -200,17 +173,17 @@
 		(cons '(nonassoc <tk> ...) (parse-precedence <ex> ...)))
 	       ((_) '())))
 	    (lalr-spec-1
-	     (syntax-rules (start expect precedence< precedence> grammar)
+	     (syntax-rules (start expect notice prec< prec> grammar)
 	       ((_ (start <symb>) <e> ...)
 		(cons (cons 'start '<symb>) (lalr-spec-1 <e> ...)))
 	       ((_ (expect <n>) <e> ...)
 		(cons (cons 'expect <n>) (lalr-spec-1 <e> ...)))
 	       ((_ (notice <str>) <e> ...)
 		(cons (cons 'notice <str>) (lalr-spec-1 <e> ...)))
-	       ((_ (precedence< <ex> ...) <e> ...)
+	       ((_ (prec< <ex> ...) <e> ...)
 		(cons (cons 'prece (reverse (parse-precedence <ex> ...)))
 		      (lalr-spec-1 <e> ...)))
-	       ((_ (precedence> <ex> ...) <e> ...)
+	       ((_ (prec> <ex> ...) <e> ...)
 		(cons (cons 'prece (parse-precedence <ex> ...))
 		      (lalr-spec-1 <e> ...)))
 	       ((_ (grammar <prod> ...) <e> ...)
@@ -305,8 +278,8 @@
 	       
   ;; fatal: non-terminal's w/o production rule		  
   (define (gram-check-3 ll nl err-l)
-    (fold-left
-     (lambda (l n)
+    (fold
+     (lambda (n l)
        (if (not (memq n ll))
 	   (cons (fmtstr "*** non-terminal with no production rule: ~A" n) l)
 	   l))
@@ -315,8 +288,8 @@
   ;; warning: unused LHS
   ;; TODO: which don't appear in OTHER RHS, e.g., (foo (foo))
   (define (gram-check-4 ll nl err-l)
-    (fold-left
-     (lambda (l s) (cons (fmtstr "+++ LHS not used in any RHS: ~A" s) l))
+    (fold
+     (lambda (s l) (cons (fmtstr "+++ LHS not used in any RHS: ~A" s) l))
      err-l
      (let iter ((ull '()) (all ll)) ; unused LHSs, all LHS's
        (if (null? all) ull
@@ -509,9 +482,9 @@
 		  (assq-ref spec 'rhs-v)
 		  '() '()))
 
-;; @item make-core/eps spec => lalr-core-type
+;; @item make-core/extras spec => lalr-core-type
 ;; Add list of symbols with epsilon productions.
-(define (make-core/eps spec)
+(define (make-core/extras spec)
   (let ((non-terms (assq-ref spec 'non-terms))
 	(terminals (assq-ref spec 'terminals))
 	(lhs-v (assq-ref spec 'lhs-v))
@@ -663,33 +636,122 @@
 	(cons p-ix r-ixm1))))
 
 ;; @item non-kernels symb => list of prule indices
-;; Compute the set of non-kernel rules for symbol @code{symb}.
-;; If A => Bcd, B => Cde, B => Abe then non-kerns A gives TBD
+;; Compute the set of non-kernel rules for symbol @code{symb}.  If grammar
+;; looks like
+;; @example
+;; 1: A => Bcd
+;; ...
+;; 5: B => Cde
+;; ...
+;; 7: B => Abe
+;; @end example
+;; @noindent
+;; then @code{non-kernels 'A} results in @code{(1 5 7)}.
+;; Note: To support pruning this routine will need to be rewritten.
 (define (non-kernels symb)
   (let* ((core (fluid-ref *lalr-core*))
 	 (lhs-v (core-lhs-v core))
 	 (glen (vector-length lhs-v))
 	 (lhs-symb (lambda (gx) (vector-ref lhs-v gx))))
-    (let iter ((rslt '())		; result is set of states
+    (let iter ((rslt '())		; result is set of p-rule indices
 	       (done '())		; symbols completed or queued
 	       (next '())		; next round of symbols to process
 	       (curr (list symb))	; this round of symbols to process
 	       (gx 0))			; p-rule index
       (cond
-       ((and (< gx glen) (memq (lhs-symb gx) curr))
-	;; Add rhs to next and rslt if not already done.
-	(let* ((rslt1 (if (memq gx rslt) rslt (cons gx rslt)))
-	       ;;(rhs1 (looking-at (cons gx 0)))
-	       (rhs1 (looking-at (first-item gx)))
-	       (done1 (if (memq rhs1 done) done (cons rhs1 done)))
-	       (next1 (cond
-		       ((memq rhs1 done) next)
-		       ((terminal? rhs1) next)
-		       (else (cons rhs1 next)))))
-	  (iter rslt1 done1 next1 curr (1+ gx))))
        ((< gx glen)
-	;; Nothing to check; process next rule.
-	(iter rslt done next curr (1+ gx)))
+	(if (memq (lhs-symb gx) curr)
+	    ;; Add rhs to next and rslt if not already done.
+	    (let* ((rhs1 (looking-at (first-item gx))) ; 1st-RHS-sym|$eps
+		   (rslt1 (if (memq gx rslt) rslt (cons gx rslt)))
+		   (done1 (if (memq rhs1 done) done (cons rhs1 done)))
+		   (next1 (cond ((memq rhs1 done) next)
+				((terminal? rhs1) next)
+				(else (cons rhs1 next)))))
+	      (iter rslt1 done1 next1 curr (1+ gx)))
+	    ;; Nothing to check; process next rule.
+	    (iter rslt done next curr (1+ gx))))
+       ((pair? next)
+	;; Start another sweep throught the grammar.
+	(iter rslt done '() next 0))
+       (else
+	;; Done, so return.
+	(reverse rslt))))))
+;; To support pruning:
+;; Maybe replace each symbol in next, done with a list, reflecting derivations.
+;; That is,
+;; 1: A  -> B | C
+;; 2: B  -> P1
+;; 3: P1 -> E, prune F
+;; 4: B  -> P2
+;; 5: P2 -> E, prune G
+;; 6: E  -> F
+;; 7: E  -> G
+;; 8: C  -> ";"
+;;
+;; curr=A next=((B A) (C A)) done=(A) rslt=(1)
+;; OR
+;; use alist of LHS to add, but prune xxx
+;; curr=((A)) next=(B C) done=(A) rslt=(1)
+;; curr=((B) (C)) next=(P1 P2) done=(A B C) rslt=(1 2 4)
+;; curr=((P1 F) (P2 G)) next=((E F) (E G))
+;;     done=(A B C P1 P2) rslt=(1 2 4 3 5)
+;; curr=((P1 F) (P2 G)) next=((E F) (E G) => (E (intersection F G))
+;; curr=((E)) next=((E F) (E G) => (E (intersection F G))
+
+;; @item p-todo symbol prunage spl
+;; updates to be handled
+;; if want stuff in spl not in E
+;; @example
+;; p-memb 'E '(F) '((E G)) => 
+;; @end example
+(define (p-todo symbol prunage spl)
+  (let* ((pnag (assq-ref symbol spl))
+	 (pset (and=> pnag (lambda (p) (lset-intersection p prunage))))
+	 )
+    (if (and pnag pset)
+	#f
+	#t)))
+(define (p-cons symbol prunage spl)
+  #f)
+(define (with-gram gram thunk)
+  (fluid-set! *lalr-core* (make-core/extras gram))
+  (thunk))
+(define (non-kernels/prunage symb)
+  (let* ((core (fluid-ref *lalr-core*))
+	 (lhs-v (core-lhs-v core))
+	 (prune (core-prune-al core))
+	 (glen (vector-length lhs-v))
+	 (lhs-symb (lambda (gx) (vector-ref lhs-v gx)))
+	 (inters lset-intersection)
+	 )
+    (let iter ((rslt '())		; result is set of p-rule indices
+	       (done '())		; symbols completed or queued
+	       (next '())		; next round of symbols to process
+	       (curr (list (list symb))) ; this round of symbols to process
+	       (gx 0))			; p-rule index
+      (cond
+       ((< gx glen)
+	(if (assq (lhs-symb gx) curr)
+	    (let* ((pair (assq (lhs-symb gx) curr))
+		   (rhs1 (looking-at (first-item gx))) ; 1st-RHS-sym | $eps
+		   (pnge (assq-ref prune rhs1))	      ; prunage, AA null
+		   (rhsx (assq rhs1 prune))
+		   ;;
+		   (rslt1 (if (memq gx rslt) rslt (cons gc rslt)))
+		   (done1 (if (assq rhs1 done)
+			      (cons 
+			       (cons rhs1 (inters (assq-ref done rhs1) rhsx))
+			       done)
+			      done))
+		   (next1 (cond ((memq rhs1 done) next)
+				((terminal? rhs1) next)
+				(else (cons rhs1 next))))
+		   
+		   )
+	      (iter rslt1 done1 next1 curr (1+ gx)))
+	    ;; Nothing to check; process next rule.
+	    (iter rslt done next curr (1+ gx))))
        ((pair? next)
 	;; Start another sweep throught the grammar.
 	(iter rslt done '() next 0))
@@ -697,16 +759,11 @@
 	;; Done, so return.
 	(reverse rslt))))))
 
-;; @item non-kernel-items symb
-;; Compute the set of non-kernel states for symbol @code{symb}.
-(define (non-kernel-items symb)
-  (map (lambda (gx) (cons gx 0)) (non-kernels symb)))
-
 ;; @item expand-k-item => item-set
 ;; Expand a kernel-item into a list with the non-kernels.
 (define (expand-k-item k-item)
   (reverse
-   (fold-left (lambda (items gx) (cons (first-item gx) items))
+   (fold (lambda (gx items) (cons (first-item gx) items))
 	      (list k-item)
 	      (non-kernels (looking-at k-item)))))
 
@@ -770,11 +827,11 @@
 ;; and @code{'kix-v}, repspectively.   Each entry in @code{kis-v} is a list of
 ;; items in the form @code{(px . rx)} where @code{px} is the production rule
 ;; index and @code{rx} is the index of the RHS symbol.  Each entry in the
-;; vector @code{kix-v} is an a-list of the form @code{sy . kx} where @code{sy}
-;; is a (terminal or non-terminal) symbol and @code{kx} is the index of the
-;; kernel itemset.  The basic algorithm is discussed on pp. 228-229 of the
-;; DB except that we compute non-kernel items on the fly using
-;; @code{expand-k-item}.  See Example 4.46 on p. 241 of the DB.
+;; vector @code{kix-v} is an a-list with entries @code{(sy . kx)} where
+;; @code{sy} is a (terminal or non-terminal) symbol and @code{kx} is the
+;; index of the kernel itemset.  The basic algorithm is discussed on
+;; pp. 228-229 of the DB except that we compute non-kernel items on the fly
+;; using @code{expand-k-item}.  See Example 4.46 on p. 241 of the DB.
 (define (step1 . rest)
   (let* ((al-in (if (pair? rest) (car rest) '()))
 	 (add-kset (lambda (upd kstz)	; give upd a ks-ix and add to kstz
@@ -860,16 +917,15 @@
 (define (first symbol-list end-token-list)
   (let* ((core (fluid-ref *lalr-core*))
 	 (eps-l (core-eps-l core))
-	 (prune-al (core-prune-al core)) ; alist of (symb . prunage-list)
-	 (stng symbol-list)
-	 (latoks end-token-list))
+	 (prune-al (core-prune-al core)) ; alist: lhs -> prune-l
+	 )
     ;; This loop strips off the leading symbol from stng and then adds to
     ;; todo list, which results in range of p-rules getting checked for
     ;; terminals.
     (let iter ((rslt '())		; terminals collected
 	       (pngl '())		; prunage-list: may need stack
-	       (stng stng)		; what's left of input string
-	       (hzeps #t)		; has epsilon production
+	       (stng symbol-list)	; what's left of input string
+	       (hzeps #t)		; if eps-prod so far
 	       (done '())		; non-terminals checked
 	       (todo '())		; non-terminals to assess
 	       (p-range '())		; range of p-rules to check
@@ -922,9 +978,10 @@
 		   (or (eq? symb '$epsilon) (memq symb eps-l))
 		   symbl symbl '() '()))))
       (hzeps
-       ;; $epsilon passes all the way through.  If latoks provided use that.
-       (if (pair? latoks)
-	   (lset-union eqv? rslt latoks)
+       ;; $epsilon passes all the way through.
+       ;; If end-token-list provided use that.
+       (if (pair? end-token-list)
+	   (lset-union eqv? rslt end-token-list)
 	   (cons '$epsilon rslt)))
       (else
        rslt)))))
@@ -1053,10 +1110,10 @@
 		  (item1 (next-item item)) ; next item after sym
 		  (sx1 (assq-ref (vector-ref kix-v kx) sym)) ; goto(I,sym)
 		  (item0 (car kset)))	   ; kernel item
-	     (kit-add kit-v (remq '$@ tokl) sx1 item1) ; spontaneous
+	     (kit-add kit-v (delq '$@ tokl) sx1 item1) ; spontaneous
 	     (if (memq '$@ tokl)	; propagates
 		 (kip-add kip-v kx item0 sx1 item1))))
-	 (remp ;; todo: check this remove
+	 (remove ;; todo: check this remove
 	  (lambda (li) (last-item? (car li)))
 	  (closure (list (cons (car kset) '($@))))))
 	(iter kx (cdr kset)))
@@ -1064,7 +1121,7 @@
        ((< (1+ kx) nkset)
 	(iter (1+ kx)
 	      ;; End-items don't shift, so don't propagate.
-	      (remp last-item? (vector-ref kis-v (1+ kx)))))))
+	      (remove last-item? (vector-ref kis-v (1+ kx)))))))
     (when #f (pp-kip-v kip-v) (pp-kit-v kit-v)) ; for debugging
     (cons* (cons 'kit-v kit-v) (cons 'kip-v kip-v) subm)))
 
@@ -1246,7 +1303,7 @@
 	  ;; the parser using precedence/associativity rules so we remove the
 	  ;; reduction from the reduction-list.
 	  (iter (cons (cons* term 'srconf goto (car redl)) res)
-		(cdr sal) (remove redp ral)))
+		(cdr sal) (delete redp ral)))
 	 (else
 	  ;; The terminal (aka token) signals a shift only.
 	  (iter (cons (cons* term 'shift goto) res)
@@ -1543,7 +1600,7 @@
 (define (make-lalr-machine spec)
   (let ((prev-core (fluid-ref *lalr-core*)))
     (dynamic-wind
-	(lambda () (fluid-set! *lalr-core* (make-core/eps spec)))
+	(lambda () (fluid-set! *lalr-core* (make-core/extras spec)))
 	(lambda ()
 	  ;; build up "sub-machines"
 	  (let* ((sm1 (step1 spec))
@@ -1825,11 +1882,18 @@
   (let* ((len-v (assq-ref mach 'len-v))
 	 (rto-v (assq-ref mach 'rto-v))	; reduce to
 	 (pat-v (assq-ref mach 'pat-v))
-	 (sya-v (vector-map ;; symbolic action
-		 (lambda (ix nrg guts) (wrap-action nrg guts))
-		 (assq-ref mach 'nrg-v) (assq-ref mach 'act-v)))
-	 ;; Turn symbolic action into executable procedures:
-	 (act-v (vector-map (lambda (ix f) (eval f (current-module))) sya-v))
+	 (act-v (cond
+		 ((assq-ref mach 'nrg-v) =>
+		  (lambda (v)
+		    (vector-map
+		     ;; Turn symbolic action into executable procedures:
+		     (lambda (ix f) (eval f (current-module)))
+		     (vector-map
+		      ;; symbolic action
+		      (lambda (ix nrg guts) (wrap-action nrg guts))
+		      (assq-ref mach 'nrg-v) (assq-ref mach 'act-v)))))
+		 (else ;; have tables: no conversion needed
+		  (assq-ref mach 'act-v))))
 	 ;;
 	 (dmsg (lambda (s t a) (fmtout "state ~S, token ~S\t=> ~S\n" s t a)))
 	 (hashed (number? (caar (vector-ref pat-v 0)))) ; been hashified?
@@ -1892,10 +1956,53 @@
 	   (else ;; accept
 	    (car stack))))))))
 
+#;(define hashed-parser-code
+  '(define* (lexr #:key debug)
+     (define (dmsg s t a) (fmtout "state ~S, token ~S\t=> ~S\n" s t a))
+     (let iter ((state (list 0)) (stack (list '$@)) (nval #f) (lval (lexr)))
+       (let* ((tval (car (if nval nval lval)))
+	      (sval (cdr (if nval nval lval)))
+	      (stxl (vector-ref pat-v (car state)))
+	      (stx (or (assq-ref stxl tval) (assq-ref stxl -1) #f)))
+	 (when debug (dmsg (car state) (if nval tval sval) stx))
+	 (cond
+	  ((eq? #f stx)
+	   (let ((fn (or (port-filename (current-input-port)) "(unknown)"))
+		 (ln (1+ (port-line (current-input-port)))))
+	     (fmterr "~A: ~A: parse failed at state ~A, on input ~S\n"
+		     fn ln (car state) sval))
+	   #f)
+	  ((positive? stx)
+	   (iter (cons (shift-to stx) state) (cons sval stack)
+		 #f (if nval lval (lexr))))
+	  ((negative? stx)
+	   (let* ((gx (abs stx)) (gl (vector-ref len-v gx))
+		  ($$ (apply (vector-ref act-v gx) stack)))
+	     (iter (list-tail state gl) 
+		   (list-tail stack gl)
+		   (cons (vector-ref rto-v gx) $$)
+		   lval)))
+	  (else (car stack)))))))
+
 
 (use-modules (ice-9 pretty-print))
 (use-modules (ice-9 regex))
 
+(define (write-notice mach port)
+  (let* ((comm-leader ";; ")
+	 (notice (assq-ref (assq-ref mach 'attr) 'notice))
+	 (lines (if notice (string-split notice #\newline) '())))
+    (for-each
+     (lambda (l) (fmt port "~A~A\n" comm-leader l))
+     lines)
+    (if (pair? lines) (newline port))))
+
+(define (string-sub str pat repl)
+  (let ((m (string-match pat str)))
+    (if m
+	(regexp-substitute #f m 'pre repl 'post)
+	str)))
+
 ;; @item write-lalr-tables mach filename [#:lang output-lang]
 ;; For example,
 ;; @example
@@ -1903,15 +2010,6 @@
 ;; write-lalr-tables mach "tables.tcl" #:lang 'tcl
 ;; @end example
 (define* (write-lalr-tables mach filename #:key (lang 'scheme))
-  (define comm-leader ";; ")
-
-  (define (write-notice mach port)
-    (let* ((notice (assq-ref (assq-ref mach 'attr) 'notice))
-	   (lines (if notice (string-split notice #\newline) '())))
-      (for-each
-       (lambda (l) (fmt port "~A~A\n" comm-leader l))
-       lines)
-      (if (pair? lines) (newline port))))
 
   (define (write-table mach name port)
     (fmt port "(define ~A\n  " name)
@@ -1954,23 +2052,53 @@
       (write-table mach 'pat-v port)
       (write-table mach 'rto-v port)
       (write-table mach 'mtab port)
+      ;;(write-actions mach port)
+      (display ";;; end tables" port)
+      (newline port))))
+
+(define* (write-lalr-actions mach filename #:key (lang 'scheme))
+  (define (pp-rule/ts gx)
+    (let* ((core (fluid-ref *lalr-core*))
+	   (lhs (vector-ref (core-lhs-v core) gx))
+	   (rhs (vector-ref (core-rhs-v core) gx))
+	   (tl (core-terminals core))
+	   (line (string-append
+		  (symbol->string lhs) " => "
+		  (string-join 
+		   (map (lambda (elt) (elt->str elt tl))
+			(vector->list rhs))
+		   " "))))
+      (if (> (string-length line) 72)
+	  (string-append (substring/shared line 0 69) "...")
+	  line)))
+    
+  (define (write-actions mach port)
+    (with-fluid*
+     *lalr-core* (make-core mach)
+     (lambda ()
+       (fmt port "(define act-v\n  (vector\n")
+       (vector-for-each
+	(lambda (ix nrg act)
+	  (fmt port "   ;; ~A\n" (pp-rule/ts ix))
+	  (pretty-print (wrap-action nrg act) port #:per-line-prefix "   "))
+	(assq-ref mach 'nrg-v) (assq-ref mach 'act-v))
+       (fmt port "   ))\n\n"))))
+
+  (call-with-output-file filename
+    (lambda (port)
+      #;(fmt port ";; ~A\n\n"
+      (regexp-substitute #f (string-match ".new$" filename) 'pre))
+      (fmt port ";; ~A\n\n" (string-sub filename ".new$" ""))
+      (write-notice mach port)
       (write-actions mach port)
       (display ";;; end tables" port)
       (newline port))))
 
-#;(define (write-lalr-tables-in-tcl mach filename)
-
-  (define (write-table mach name port)
-    (fmt port "(define ~A\n  " name)
-    (ugly-print-to-tcl (assq-ref mach name) port)
-    (fmt port ")\n\n"))
-			
+#;(define* (write-lalr-parser mach filename #:key (lang 'scheme))
   (call-with-output-file filename
     (lambda (port)
-      (fmt port ";; ~A\n\n"
-	   (regexp-substitute #f (string-match ".new$" filename) 'pre))
-      (write-table mach 'len-v port)
-      )))
+      (pretty-print hashed-parser-code port)
+)))
 
 ;; @end itemize
 
